@@ -180,7 +180,7 @@ static void GetHostname (
 	return;
 
     buf[0] = '\0';
-    (void) gethostname (buf, maxlen);
+    (void) gethostname (buf, (size_t) maxlen);
     buf [maxlen - 1] = '\0';
 #endif
 }
@@ -268,7 +268,7 @@ void _XtInherit(void)
 {
     XtErrorMsg("invalidProcedure","inheritanceProc",XtCXtToolkitError,
             "Unresolved inheritance operation",
-              (String *)NULL, (Cardinal *)NULL);
+              NULL, NULL);
 }
 
 
@@ -301,7 +301,7 @@ void XtToolkitInitialize(void)
 
 
 String _XtGetUserName(
-    String dest,
+    _XtString dest,
     int len)
 {
 #ifdef WIN32
@@ -320,11 +320,11 @@ String _XtGetUserName(
     char* ptr;
 
     if ((ptr = getenv("USER"))) {
-	(void) strncpy (dest, ptr, len-1);
+	(void) strncpy (dest, ptr, (size_t) (len-1));
 	dest[len-1] = '\0';
     } else {
 	if ((pw = _XGetpwuid(getuid(),pwparams)) != NULL) {
-	    (void) strncpy (dest, pw->pw_name, len-1);
+	    (void) strncpy (dest, pw->pw_name, (size_t)(len-1));
 	    dest[len-1] = '\0';
 	} else
 	    *dest = '\0';
@@ -335,7 +335,7 @@ String _XtGetUserName(
 
 
 static String GetRootDirName(
-    String dest,
+    _XtString dest,
     int len)
 {
 #ifdef WIN32
@@ -360,22 +360,22 @@ static String GetRootDirName(
 #ifdef X_NEEDS_PWPARAMS
     _Xgetpwparams pwparams;
 #endif
-    struct passwd *pw;
     static char *ptr;
 
     if (len <= 0 || dest == NULL)
 	return NULL;
 
     if ((ptr = getenv("HOME"))) {
-	(void) strncpy (dest, ptr, len-1);
+	(void) strncpy (dest, ptr, (size_t)(len-1));
 	dest[len-1] = '\0';
     } else {
+        struct passwd *pw;
 	if ((ptr = getenv("USER")))
 	    pw = _XGetpwnam(ptr,pwparams);
 	else
  	    pw = _XGetpwuid(getuid(),pwparams);
 	if (pw != NULL) {
-	    (void) strncpy (dest, pw->pw_dir, len-1);
+	    (void) strncpy (dest, pw->pw_dir, (size_t)(len-1));
 	    dest[len-1] = '\0';
 	} else
 	    *dest = '\0';
@@ -425,19 +425,19 @@ static void CombineUserDefaults(
     Display *dpy,
     XrmDatabase *pdb)
 {
-#ifdef __MINGW32__
-    const char *slashDotXdefaults = "/Xdefaults";
-#else
-    const char *slashDotXdefaults = "/.Xdefaults";
-#endif
     char *dpy_defaults = XResourceManagerString(dpy);
 
     if (dpy_defaults) {
 	XrmCombineDatabase(XrmGetStringDatabase(dpy_defaults), pdb, False);
     } else {
+#ifdef __MINGW32__
+	const char *slashDotXdefaults = "/Xdefaults";
+#else
+	const char *slashDotXdefaults = "/.Xdefaults";
+#endif
 	char filename[PATH_MAX];
 	(void) GetRootDirName(filename,
-			PATH_MAX - strlen (slashDotXdefaults) - 1);
+			PATH_MAX - (int)strlen (slashDotXdefaults) - 1);
 	(void) strcat(filename, slashDotXdefaults);
 	(void)XrmCombineFileDatabase(filename, pdb, False);
     }
@@ -574,9 +574,9 @@ XrmDatabase XtScreenDatabase(
 #endif
 
 	    (void) GetRootDirName(filename = filenamebuf,
-			PATH_MAX - strlen (slashDotXdefaultsDash) - 1);
+			PATH_MAX - (int)strlen (slashDotXdefaultsDash) - 1);
 	    (void) strcat(filename, slashDotXdefaultsDash);
-	    len = strlen(filename);
+	    len = (int)strlen(filename);
 	    GetHostname (filename+len, PATH_MAX-len);
 	}
 	(void)XrmCombineFileDatabase(filename, &db, False);
@@ -648,24 +648,26 @@ static void _MergeOptionTables(
     Cardinal *num_dst)
 {
     XrmOptionDescRec *table, *endP;
-    register XrmOptionDescRec *opt1, *whereP, *dstP;
-    register const XrmOptionDescRec *opt2;
+    XrmOptionDescRec *opt1, *dstP;
+    const XrmOptionDescRec *opt2;
     int i1;
     Cardinal i2;
     int dst_len, order;
-    Boolean found;
     enum {Check, NotSorted, IsSorted} sort_order = Check;
 
     *dst = table = (XrmOptionDescRec*)
-	__XtMalloc( sizeof(XrmOptionDescRec) * (num_src1 + num_src2) );
+	__XtMalloc( (Cardinal)(sizeof(XrmOptionDescRec) * (num_src1 + num_src2) ));
 
     (void) memmove(table, src1, sizeof(XrmOptionDescRec) * num_src1 );
     if (num_src2 == 0) {
 	*num_dst = num_src1;
 	return;
     }
-    endP = &table[dst_len = num_src1];
+    endP = &table[dst_len = (int)num_src1];
     for (opt2 = src2, i2= 0; i2 < num_src2; opt2++, i2++) {
+        XrmOptionDescRec *whereP;
+        Boolean found;
+
 	found = False;
 	whereP = endP-1;	/* assume new option goes at the end */
 	for (opt1 = table, i1 = 0; i1 < dst_len; opt1++, i1++) {
@@ -709,7 +711,7 @@ static void _MergeOptionTables(
 	    dst_len++;
 	}
     }
-    *num_dst = dst_len;
+    *num_dst = (Cardinal)dst_len;
 }
 
 
@@ -737,7 +739,7 @@ static Boolean _GetResource(
 	    return True;
 	} else {
 	    WidgetRec widget; /* hack, hack */
-	    bzero( &widget, sizeof(widget) );
+	    memset( &widget, 0, sizeof(widget) );
 	    widget.core.self = &widget;
 	    widget.core.widget_class = coreWidgetClass;
 	    widget.core.screen = (Screen*)DefaultScreenOfDisplay(dpy);
@@ -754,7 +756,7 @@ XrmDatabase _XtPreparseCommandLine(
     XrmOptionDescRec *urlist,
     Cardinal num_urs,
     int argc,
-    String *argv,
+    _XtString *argv,
     /* return */
     String *applName,
     String *displayName,
@@ -767,16 +769,16 @@ XrmDatabase _XtPreparseCommandLine(
     XrmName class_list[3];
     XrmRepresentation type;
     XrmValue val;
-    String *targv;
+    _XtString *targv;
     int targc = argc;
 
-    targv = (String *) __XtMalloc(sizeof(char *) * argc);
-    (void) memmove(targv, argv, sizeof(char *) * argc);
+    targv = (_XtString *) __XtMalloc((Cardinal)(sizeof(_XtString*) * (size_t)argc));
+    (void) memmove(targv, argv, sizeof(char *) * (size_t) argc);
     _MergeOptionTables(opTable, XtNumber(opTable), urlist, num_urs,
 		       &options, &num_options);
     name_list[0] = class_list[0] = XrmPermStringToQuark(".");
     name_list[2] = class_list[2] = NULLQUARK;
-    XrmParseCommand(&db, options, num_options, ".", &targc, targv);
+    XrmParseCommand(&db, options, (int) num_options, ".", &targc, targv);
     if (applName) {
 	name_list[1] = XrmPermStringToQuark("name");
 	if (XrmQGetResource(db, name_list, name_list, &type, &val) &&
@@ -893,7 +895,7 @@ void _XtDisplayInitialize(
 	/* Parse the command line and remove Xt arguments from argv */
 	_MergeOptionTables( opTable, XtNumber(opTable), urlist, num_urs,
 			    &options, &num_options );
-	XrmParseCommand(&pd->cmd_db, options, num_options, name, argc, argv);
+	XrmParseCommand(&pd->cmd_db, options, (int) num_options, name, argc, argv);
 
 	db = XtScreenDatabase(DefaultScreenOfDisplay(dpy));
 
@@ -908,7 +910,7 @@ void _XtDisplayInitialize(
 	while (!XrmQGetSearchList(db, name_list, class_list,
 				  search_list, search_list_size)) {
 	    XrmHashTable* old = search_list;
-	    Cardinal size = (search_list_size*=2)*sizeof(XrmHashTable);
+	    Cardinal size = (Cardinal) ((size_t)(search_list_size *= 2)*sizeof(XrmHashTable));
 	    if (!(search_list = (XrmHashTable*)ALLOCATE_LOCAL(size)))
 		_XtAllocError(NULL);
 	    (void) memmove((char*)search_list, (char*)old, (size>>1) );
@@ -985,7 +987,7 @@ XtAppSetFallbackResources(
 Widget XtOpenApplication(XtAppContext *app_context_return,
 			 _Xconst char *application_class,
 			 XrmOptionDescRec *options, Cardinal num_options,
-			 int *argc_in_out, String *argv_in_out,
+			 int *argc_in_out, _XtString *argv_in_out,
 			 String *fallback_resources, WidgetClass widget_class,
 			 ArgList args_in, Cardinal num_args_in)
 {
@@ -1029,7 +1031,7 @@ XtAppInitialize(
     XrmOptionDescRec *options,
     Cardinal num_options,
     int *argc_in_out,
-    String *argv_in_out,
+    _XtString *argv_in_out,
     String *fallback_resources,
     ArgList args_in,
     Cardinal num_args_in)
@@ -1045,12 +1047,12 @@ XtAppInitialize(
 /*ARGSUSED*/
 Widget
 XtInitialize(
-    _Xconst char* name,
-    _Xconst char* classname,
+    _Xconst _XtString name,
+    _Xconst _XtString classname,
     XrmOptionDescRec *options,
     Cardinal num_options,
     int *argc,
-    String *argv)
+    _XtString *argv)
 {
     Widget root;
     XtAppContext app_con;
